﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.IO;

namespace howto_draw_curves2
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // True when we are drawing.
        private bool Drawing = true;

        // The currently selected points.
        private List<Point> Points = new List<Point>();

        // The curve's tension.
        private float Tension = 0.5f;

        // The user clicked. Add a point,
        // stop drawing, or start a new curve.
        private void picCanvas_MouseClick(object sender, MouseEventArgs e)
        {
            // See if we are currently drawing.
            if (Drawing)
            {
                // See if this is the left or right mouse button.
                if (e.Button == MouseButtons.Left)
                {
                    // Left button. Add a new point.
                    Points.Add(e.Location);
                }
                else
                {
                    // Right button. Stop drawing.
                    Drawing = false;
                }
            }
            else
            {
                // We are not drawing. Start a new curve.
                Drawing = true;
                Points = new List<Point>();

                // Add a new point.
                Points.Add(e.Location);
            }

            // Redraw.
            picCanvas.Refresh();
        }

        // Draw the curve and its points.
        private void picCanvas_Paint(object sender, PaintEventArgs e)
        {
            DrawTheCurve(e.Graphics, true);
        }

        // Draw the curve.
        private void DrawTheCurve(Graphics gr, bool draw_points)
        {
            gr.SmoothingMode = SmoothingMode.AntiAlias;

            // Draw the curve.
            if (Points.Count > 1)
            {
                // Make a pen to use.
                using (Pen pen = new Pen(Color.Blue))
                {
                    // See if we're currently drawing.
                    if (Drawing)
                    {
                        // Use a dashed pen.
                        pen.DashPattern = new float[] { 5, 5 };
                    }

                    // Draw the curve.
                    gr.DrawCurve(pen, Points.ToArray(), Tension);
                }
            }

            // Draw the points.
            if (draw_points && Drawing && (Points.Count > 0))
            {
                const int r = 4;
                foreach (Point point in Points)
                {
                    Rectangle rect = new Rectangle(
                        point.X - r, point.Y - r, 2 * r, 2 * r);
                    gr.FillRectangle(Brushes.White, rect);
                    gr.DrawRectangle(Pens.Black, rect);
                }
            }
        }

        // Adjust the curve's tension.
        private void scrTension_Scroll(object sender, ScrollEventArgs e)
        {
            Tension = scrTension.Value / 10f;
            lblTension.Text = Tension.ToString();
            picCanvas.Refresh();
        }

        // Save an image of the PictureBox.
        private void mnuFileSaveImage_Click(object sender, EventArgs e)
        {
            if (sfdPicture.ShowDialog() == DialogResult.OK)
            {
                // Get an image of the PictureBox.
                using (Bitmap bm = GetControlImage(picCanvas))
                {
                    // Save the image.
                    SaveImage(bm, sfdPicture.FileName);
                }
            }
        }

        // Draw the curve onto a bitmap and save it.
        private void mnuFileSaveDrawing_Click(object sender, EventArgs e)
        {
            if (sfdPicture.ShowDialog() == DialogResult.OK)
            {
                // Make a Bitmap.
                int wid = picCanvas.ClientSize.Width;
                int hgt = picCanvas.ClientSize.Height;
                using (Bitmap bm = new Bitmap(wid, hgt))
                {
                    // Draw the curve on the bitmap.
                    using (Graphics gr = Graphics.FromImage(bm))
                    {
                        DrawTheCurve(gr, false);
                    }

                    // Save the image.
                    SaveImage(bm, sfdPicture.FileName);
                }
            }
        }

        private void mnuFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        // Return a Bitmap holding an image of the control.
        // See: http://csharphelper.com/blog/2014/09/get-the-image-of-a-control-or-form-or-a-forms-client-area-in-c/
        private Bitmap GetControlImage(Control ctl)
        {
            Bitmap bm = new Bitmap(ctl.Width, ctl.Height);
            ctl.DrawToBitmap(bm,
                new Rectangle(0, 0, ctl.Width, ctl.Height));
            return bm;
        }

        // Save the file with the appropriate format.
        // See http://csharphelper.com/blog/2014/07/save-images-with-an-appropriate-format-depending-on-the-file-names-extension-in-c/
        public void SaveImage(Image image, string filename)
        {
            string extension = Path.GetExtension(filename);
            switch (extension.ToLower())
            {
                case ".bmp":
                    image.Save(filename, ImageFormat.Bmp);
                    break;
                case ".exif":
                    image.Save(filename, ImageFormat.Exif);
                    break;
                case ".gif":
                    image.Save(filename, ImageFormat.Gif);
                    break;
                case ".jpg":
                case ".jpeg":
                    image.Save(filename, ImageFormat.Jpeg);
                    break;
                case ".png":
                    image.Save(filename, ImageFormat.Png);
                    break;
                case ".tif":
                case ".tiff":
                    image.Save(filename, ImageFormat.Tiff);
                    break;
                default:
                    throw new NotSupportedException(
                        "Unknown file extension " + extension);
            }
        }
    }
}
